@@ -1,5 +1,5 @@
 ## routines for very large dataset generalized additive modelling.
-## (c) Simon N. Wood 2009-2012
+## (c) Simon N. Wood 2009-2013
 
 
 ls.size <- function(x) {
@@ -17,11 +17,11 @@ ls.size <- function(x) {
 
 rwMatrix <- function(stop,row,weight,X) {
 ## Routine to recombine the rows of a matrix X according to info in 
-## stop,row and weight. Consider the ith row of the output matrix 
+## stop, row and weight. Consider the ith row of the output matrix 
 ## ind <- 1:stop[i] if i==1 and ind <- (stop[i-1]+1):stop[i]
 ## otherwise. The ith output row is then X[row[ind],]*weight[ind]
   if (is.matrix(X)) { n <- nrow(X);p<-ncol(X);ok <- TRUE} else { n<- length(X);p<-1;ok<-FALSE}
-  stop <- stop - 1;row <- row - 1 ## R indeces -> C indeces
+  stop <- stop - 1;row <- row - 1 ## R indices -> C indices
   oo <-.C(C_rwMatrix,as.integer(stop),as.integer(row),as.double(weight),X=as.double(X),as.integer(n),as.integer(p))
   if (ok) return(matrix(oo$X,n,p)) else
   return(oo$X) 
@@ -31,16 +31,29 @@ chol2qr <- function(XX,Xy) {
 ## takes X'X and X'y and returns R and f
 ## equivalent to qr update. Uses simple
 ## regularization if XX not +ve def. 
+
+  #d <- diag(XX)
+  #ind <- d>0
+  #d[!ind] <- 1;d[ind] <- 1/sqrt(d[ind])
+  #XX <- t(d*t(d*XX)) ## diagonal pre-conditioning
+
   XXeps <- norm(XX)*.Machine$double.eps^.9
   n <- ncol(XX)
   for (i in 0:20) {
     ok <- TRUE
-    R <- try(chol(XX+diag(n)*XXeps*i),silent=TRUE) ## R'R = X'X
+    R <- try(chol(XX+diag(n)*XXeps*i,pivot=TRUE),silent=TRUE) ## R'R = X'X
     if (inherits(R,"try-error")) ok <- FALSE else {
-      f <- try(forwardsolve(t(R),Xy),silent=TRUE)
+      ipiv <- piv <- attr(R,"pivot") 
+      #f <- try(forwardsolve(t(R),(Xy*d)[piv]),silent=TRUE) 
+      f <- try(forwardsolve(t(R),Xy[piv]),silent=TRUE)
       if (inherits(f,"try-error")) ok <- FALSE
     }
-    if (ok) break; ## success
+    if (ok) { 
+      ipiv[piv] <- 1:ncol(R)
+      #R <- t(t(R[,ipiv])/d)
+      R <- R[,ipiv]
+      break; ## success
+    }
   }
   if (i==20 && !ok) stop("Choleski based method failed, switch to QR")
   list(R=R,f=f)
@@ -110,8 +123,13 @@ qr.up <- function(arg) {
 mini.mf <-function(mf,chunk.size) {
 ## takes a model frame and produces a representative subset of it, suitable for 
 ## basis setup.
+  ## first count the minimum number of rows required for representiveness
+  mn <- 0
+  for (j in 1:length(mf)) mn <- mn + if (is.factor(mf[[j]])) length(levels(mf[[j]])) else 2
+  if (chunk.size < mn) chunk.size <- mn
   n <- nrow(mf)
-  if (n<=chunk.size) return(mf)
+  if (n <= chunk.size) return(mf)
+
   seed <- try(get(".Random.seed",envir=.GlobalEnv),silent=TRUE) ## store RNG seed
   if (inherits(seed,"try-error")) {
      runif(1)
@@ -119,25 +137,60 @@ mini.mf <-function(mf,chunk.size) {
   }
   kind <- RNGkind(NULL)
   RNGkind("default", "default")
-  set.seed(66)
+  set.seed(66)  
+  ## randomly sample from original frame...
   ind <- sample(1:n,chunk.size)
-  RNGkind(kind[1], kind[2])
-  assign(".Random.seed", seed, envir = .GlobalEnv)
-  mf0 <- mf[ind,] ## random sample of rows
-  ## now need to ensure that max and min are in sample for each element of mf0
-  ## note that min and max might occur twice, but this shouldn't matter (and
-  ## is better than min overwriting max, for example)
+  mf0 <- mf[ind,]
+  ## ... now need to ensure certain sorts of representativeness
+
+  ## work through elements collecting the rows containing 
+  ## max and min for each variable, and a random row for each 
+  ## factor level....
+
+  ind <- sample(1:n,n,replace=FALSE) ## randomized index for stratified sampling w.r.t. factor levels
+  fun <- function(X,fac,ind) ind[fac[ind]==X][1] ## stratified sampler
+  k <- 0 
   for (j in 1:length(mf)) if (is.numeric(mf0[[j]])) {
     if (is.matrix(mf0[[j]])) { ## find row containing minimum
       j.min <- min((1:n)[as.logical(rowSums(mf[[j]]==min(mf[[j]])))])
       j.max <- min((1:n)[as.logical(rowSums(mf[[j]]==max(mf[[j]])))])
-      mf0[[j]][1,] <- mf[[j]][j.min,]
-      mf0[[j]][2,] <- mf[[j]][j.max,] 
     } else { ## vector
-      mf0[[j]][1] <- min(mf[[j]])
-      mf0[[j]][2] <- max(mf[[j]]) 
+      j.min <- min(which(mf[[j]]==min(mf[[j]])))
+      j.max <- min(which(mf[[j]]==max(mf[[j]])))
     }
+    k <- k + 1; mf0[k,] <- mf[j.min,]
+    k <- k + 1; mf0[k,] <- mf[j.max,] 
+  } else if (is.factor(mf[[j]])) { ## factor variable...
+    ## randomly sample one row from each factor level...
+    find <- apply(X=as.matrix(levels(mf[[j]])),MARGIN=1,FUN=fun,fac=mf[[j]],ind=ind)
+    nf <- length(find)
+    mf0[(k+1):(k+nf),] <- mf[find,]
+    k <- k + nf
   }
+
+  RNGkind(kind[1], kind[2])
+  assign(".Random.seed", seed, envir = .GlobalEnv)
+
+## problems with the following are...
+## 1. you can produce model frame rows that are wholly un-representative of the 
+##    data for multi dimensional smooths this way, by pairing extreme values
+##    with values of other variables that they never occur near.
+## 2. Nothing is done to ensure that all factor levels are present.  
+#  mf0 <- mf[ind,] ## random sample of rows
+  ## now need to ensure that max and min are in sample for each element of mf0
+  ## note that min and max might occur twice, but this shouldn't matter (and
+  ## is better than min overwriting max, for example)
+#  for (j in 1:length(mf)) if (is.numeric(mf0[[j]])) {
+#    if (is.matrix(mf0[[j]])) { ## find row containing minimum
+#      j.min <- min((1:n)[as.logical(rowSums(mf[[j]]==min(mf[[j]])))])
+#      j.max <- min((1:n)[as.logical(rowSums(mf[[j]]==max(mf[[j]])))])
+#      mf0[[j]][1,] <- mf[[j]][j.min,]
+#      mf0[[j]][2,] <- mf[[j]][j.max,] 
+#    } else { ## vector
+#      mf0[[j]][1] <- min(mf[[j]])
+#      mf0[[j]][2] <- max(mf[[j]]) 
+#    }
+#  }
   mf0
 }
 
@@ -346,7 +399,8 @@ bgam.fit <- function (G, mf, chunk.size, gp ,scale ,gamma,method, coef=NULL,etas
 
       if (method=="GCV.Cp") {
          fit <- magic(qrx$f,qrx$R,G$sp,G$S,G$off,L=G$L,lsp0=G$lsp0,rank=G$rank,
-                      H=G$H,C=G$C,gamma=gamma,scale=scale,gcv=(scale<=0),
+                      H=G$H,C=matrix(0,0,ncol(qrx$R)),     ##C=G$C,
+                      gamma=gamma,scale=scale,gcv=(scale<=0),
                       extra.rss=rss.extra,n.score=nobs+nobs.extra)
  
          post <- magic.post.proc(qrx$R,fit,qrx$f*0+1) 
@@ -524,9 +578,9 @@ bgam.fit2 <- function (G, mf, chunk.size, gp ,scale ,gamma,method, etastart = NU
       dev <- sum(dev.resids(y,mu,G$w))
       W <- Diagonal(length(w),sqrt(w))
       if (sum(good)<nobs) {
-        XWX <- as(Matrix:::crossprod(W%*%X[good,]),"matrix")
+        XWX <- as(Matrix::crossprod(W%*%X[good,]),"matrix")
       } else {
-        XWX <- as(Matrix:::crossprod(W%*%X),"matrix")
+        XWX <- as(Matrix::crossprod(W%*%X),"matrix")
       }      
       qrx <- list(R = chol(XWX))
       Wz <- W%*%z
@@ -555,7 +609,8 @@ bgam.fit2 <- function (G, mf, chunk.size, gp ,scale ,gamma,method, etastart = NU
 
       if (method=="GCV.Cp") {
          fit <- magic(qrx$f,qrx$R,G$sp,G$S,G$off,L=G$L,lsp0=G$lsp0,rank=G$rank,
-                      H=G$H,C=G$C,gamma=gamma,scale=scale,gcv=(scale<=0),
+                      H=G$H,C= matrix(0,0,ncol(qrx$R)), ##C=G$C,
+                      gamma=gamma,scale=scale,gcv=(scale<=0),
                       extra.rss=rss.extra,n.score=G$n)
  
          post <- magic.post.proc(qrx$R,fit,qrx$f*0+1) 
@@ -637,11 +692,19 @@ ar.qr.up <- function(arg) {
     ind <- arg$start[i]:arg$end[i] 
     if (arg$rho!=0) { ## have to find AR1 transform...
        N <- arg$end[i]-arg$start[i]+1
-       ## not first row implied by this transform
+       ## note first row implied by this transform
        ## is always dropped, unless really at beginning of data.
        row <- c(1,rep(1:N,rep(2,N))[-c(1,2*N)])
        weight <- c(1,rep(c(sd,ld),N-1))
        stop <- c(1,1:(N-1)*2+1)
+       if (!is.null(arg$mf$"(AR.start)")) { ## need to correct the start of new AR sections...
+           ii <- which(arg$mf$"(AR.start)"[ind]==TRUE)
+           if (length(ii)>0) {
+             if (ii[1]==1) ii <- ii[-1] ## first observation does not need any correction
+             weight[ii*2-2] <- 0 ## zero sub diagonal
+             weight[ii*2-1] <- 1 ## set leading diagonal to 1
+           }
+       }
      } 
      ## arg$G$model <- arg$mf[ind,]
      w <- sqrt(arg$G$w[ind])
@@ -744,7 +807,8 @@ predict.bam <- function(object,newdata,type="link",se.fit=FALSE,terms=NULL,
   }
 } ## end predict.bam 
 
-bam.fit <- function(G,mf,chunk.size,gp,scale,gamma,method,rho=0,cl=NULL,gc.level=0,use.chol=FALSE) 
+bam.fit <- function(G,mf,chunk.size,gp,scale,gamma,method,rho=0,
+                    cl=NULL,gc.level=0,use.chol=FALSE) 
 ## function that does big additive model fit in strictly additive case
 {  ## first perform the QR decomposition, blockwise....
    n <- nrow(mf)
@@ -822,7 +886,15 @@ bam.fit <- function(G,mf,chunk.size,gp,scale,gamma,method,rho=0,cl=NULL,gc.level
 
            row <- c(1,rep(1:N,rep(2,N))[-c(1,2*N)])
            weight <- c(1,rep(c(sd,ld),N-1))
-           stop <- c(1,1:(N-1)*2+1)
+           stop <- c(1,1:(N-1)*2+1) 
+           if (!is.null(mf$"(AR.start)")) { ## need to correct the start of new AR sections...
+             ii <- which(mf$"(AR.start)"[ind]==TRUE)
+             if (length(ii)>0) {
+               if (ii[1]==1) ii <- ii[-1] ## first observation does not need any correction
+               weight[ii*2-2] <- 0 ## zero sub diagonal
+               weight[ii*2-1] <- 1 ## set leading diagonal to 1
+             }
+           }
          } 
          #G$model <- mf[ind,]
          w <- sqrt(G$w[ind])
@@ -888,6 +960,14 @@ bam.fit <- function(G,mf,chunk.size,gp,scale,gamma,method,rho=0,cl=NULL,gc.level
        row <- c(1,rep(1:n,rep(2,n))[-c(1,2*n)])
        weight <- c(1,rep(c(sd,ld),n-1))
        stop <- c(1,1:(n-1)*2+1)
+       if (!is.null(mf$"(AR.start)")) { ## need to correct the start of new AR sections...
+         ii <- which(mf$"(AR.start)"==TRUE)
+         if (length(ii)>0) {
+           if (ii[1]==1) ii <- ii[-1] ## first observation does not need any correction
+           weight[ii*2-2] <- 0 ## zero sub diagonal
+           weight[ii*2-1] <- 1 ## set leading diagonal to 1
+         }
+       }
        yX.last <- c(G$y[n],G$X[n,])  ## store final row, in case of update
        X <- rwMatrix(stop,row,weight,sqrt(G$w)*G$X)
        y <- rwMatrix(stop,row,weight,sqrt(G$w)*G$y)
@@ -906,7 +986,8 @@ bam.fit <- function(G,mf,chunk.size,gp,scale,gamma,method,rho=0,cl=NULL,gc.level
  
    if (method=="GCV.Cp") {
      fit <- magic(qrx$f,qrx$R,G$sp,G$S,G$off,L=G$L,lsp0=G$lsp0,rank=G$rank,
-                H=G$H,C=G$C,gamma=gamma,scale=scale,gcv=(scale<=0),
+                H=G$H,C=matrix(0,0,ncol(qrx$R)),  ##C=G$C,
+                gamma=gamma,scale=scale,gcv=(scale<=0),
                 extra.rss=rss.extra,n.score=n)
  
      post <- magic.post.proc(qrx$R,fit,qrx$f*0+1) 
@@ -939,7 +1020,8 @@ bam.fit <- function(G,mf,chunk.size,gp,scale,gamma,method,rho=0,cl=NULL,gc.level
      }
      
      if (rho!=0) { ## correct RE/ML score for AR1 transform
-       object$gcv.ubre <- object$gcv.ubre - (n-1)*log(ld)
+       df <- if (is.null(mf$"(AR.start)")) 1 else sum(mf$"(AR.start)")
+       object$gcv.ubre <- object$gcv.ubre - (n-df)*log(ld)
      }
 
      G$X <- qrx$R;G$dev.extra <- rss.extra
@@ -958,8 +1040,9 @@ bam.fit <- function(G,mf,chunk.size,gp,scale,gamma,method,rho=0,cl=NULL,gc.level
      G$n.true <- n
      object <- gam(G=G,method=method,gamma=gamma,scale=scale)
      y -> G$y; w -> G$w; n -> G$n;offset -> G$offset
-     if (rho!=0) { ## correct RE/ML score for AR1 transform
-       object$gcv.ubre <- object$gcv.ubre - (n-1)*log(ld)
+     if (rho!=0) { ## correct RE/ML score for AR1 transform 
+       df <- if (is.null(mf$"(AR.start)")) 1 else sum(mf$"(AR.start)")
+       object$gcv.ubre <- object$gcv.ubre - (n-df)*log(ld)
      }
    }
    if (method=="GCV.Cp") { 
@@ -1032,7 +1115,7 @@ sparse.model.matrix <- function(G,mf,chunk.size) {
 
 bam <- function(formula,family=gaussian(),data=list(),weights=NULL,subset=NULL,na.action=na.omit,
                 offset=NULL,method="fREML",control=list(),scale=0,gamma=1,knots=NULL,
-                sp=NULL,min.sp=NULL,paraPen=NULL,chunk.size=10000,rho=0,sparse=FALSE,cluster=NULL,
+                sp=NULL,min.sp=NULL,paraPen=NULL,chunk.size=10000,rho=0,AR.start=NULL,sparse=FALSE,cluster=NULL,
                 gc.level=1,use.chol=FALSE,samfrac=1,...)
 
 ## Routine to fit an additive model to a large dataset. The model is stated in the formula, 
@@ -1061,10 +1144,10 @@ bam <- function(formula,family=gaussian(),data=list(),weights=NULL,subset=NULL,n
     method <- "REML"
     warning("sparse=TRUE not supported with fast REML, reset to REML.")
   }
-  gp<-interpret.gam(formula) # interpret the formula 
-  cl<-match.call() # call needed in gam object for update to work
-  mf<-match.call(expand.dots=FALSE)
-  mf$formula<-gp$fake.formula 
+  gp <- interpret.gam(formula) # interpret the formula 
+  cl <- match.call() # call needed in gam object for update to work
+  mf <- match.call(expand.dots=FALSE)
+  mf$formula <- gp$fake.formula 
   mf$method <-  mf$family<-mf$control<-mf$scale<-mf$knots<-mf$sp<-mf$min.sp <- mf$gc.level <-
   mf$gamma <- mf$paraPen<- mf$chunk.size <- mf$rho <- mf$sparse <- mf$cluster <-
   mf$use.chol <- mf$samfrac <- mf$...<-NULL
@@ -1076,12 +1159,15 @@ bam <- function(formula,family=gaussian(),data=list(),weights=NULL,subset=NULL,n
   pmf <- eval(pmf, parent.frame()) # pmf contains all data for parametric part
   pterms <- attr(pmf,"terms") ## pmf only used for this
   rm(pmf);
+
   if (gc.level>0) gc()
 
   mf <- eval(mf, parent.frame()) # the model frame now contains all the data 
   if (nrow(mf)<2) stop("Not enough (non-NA) data to do anything meaningful")
   terms <- attr(mf,"terms")
   if (gc.level>0) gc()  
+  if (rho!=0&&!is.null(mf$"(AR.start)")) if (!is.logical(mf$"(AR.start)")) stop("AR.start must be logical")
+
 
   ## summarize the *raw* input variables
   ## note can't use get_all_vars here -- buggy with matrices
@@ -1094,7 +1180,7 @@ bam <- function(formula,family=gaussian(),data=list(),weights=NULL,subset=NULL,n
   dl <- eval(inp, data, parent.frame())
   if (!control$keepData) { rm(data);gc()} ## save space
   names(dl) <- vars ## list of all variables needed
-  var.summary <- mgcv:::variable.summary(gp$pf,dl,nrow(mf)) ## summarize the input data
+  var.summary <- variable.summary(gp$pf,dl,nrow(mf)) ## summarize the input data
   rm(dl); if (gc.level>0) gc() ## save space    
 
   ## need mini.mf for basis setup, then accumulate full X, y, w and offset
@@ -1102,7 +1188,8 @@ bam <- function(formula,family=gaussian(),data=list(),weights=NULL,subset=NULL,n
     
   if (sparse) sparse.cons <- 2 else sparse.cons <- -1
 
-  G <- gam.setup(gp,pterms=pterms,data=mf0,knots=knots,sp=sp,min.sp=min.sp,
+  G <- gam.setup(gp,pterms=pterms,
+                 data=mf0,knots=knots,sp=sp,min.sp=min.sp,
                  H=NULL,absorb.cons=TRUE,sparse.cons=sparse.cons,select=FALSE,
                  idLinksBases=TRUE,scale.penalty=control$scalePenalty,
                  paraPen=paraPen)
@@ -1112,8 +1199,10 @@ bam <- function(formula,family=gaussian(),data=list(),weights=NULL,subset=NULL,n
 
   G$var.summary <- var.summary
   G$family <- family
-  G$terms<-terms;G$pterms<-pterms
-  
+  G$terms<-terms;##G$pterms<-pterms
+  pvars <- all.vars(delete.response(terms))
+  G$pred.formula <- if (length(pvars)>0) reformulate(pvars) else NULL
+
   n <- nrow(mf)
   
   if (is.null(mf$"(weights)")) G$w<-rep(1,n)
@@ -1122,12 +1211,12 @@ bam <- function(formula,family=gaussian(),data=list(),weights=NULL,subset=NULL,n
   G$offset <- model.offset(mf)  
   if (is.null(G$offset)) G$offset <- rep(0,n)
 
-  if (ncol(G$X)>nrow(mf)+nrow(G$C)) stop("Model has more coefficients than data")
+  if (ncol(G$X)>nrow(mf)) stop("Model has more coefficients than data")      ##+nrow(G$C)) stop("Model has more coefficients than data")
  
   G$cl<-cl;
   G$am <- am
      
-  G$min.edf<-G$nsdf-dim(G$C)[1]
+  G$min.edf<-G$nsdf #-dim(G$C)[1]
   if (G$m) for (i in 1:G$m) G$min.edf<-G$min.edf+G$smooth[[i]]$null.space.dim
 
   G$formula<-formula
@@ -1144,7 +1233,6 @@ bam <- function(formula,family=gaussian(),data=list(),weights=NULL,subset=NULL,n
   colnamesX <- colnames(G$X)  
 
   if (sparse) { ## Form a sparse model matrix...
-    require(Matrix)
     if (sum(G$X==0)/prod(dim(G$X))<.5) warning("model matrix too dense for any possible benefit from sparse")
     if (nrow(mf)<=chunk.size) G$X <- as(G$X,"dgCMatrix") else 
       G$X <- sparse.model.matrix(G,mf,chunk.size)
@@ -1214,7 +1302,7 @@ bam <- function(formula,family=gaussian(),data=list(),weights=NULL,subset=NULL,n
   object$offset <- G$offset
   object$prior.weights <- G$w
   object$pterms <- G$pterms
- 
+  object$pred.formula <- G$pred.formula
   object$smooth <- G$smooth
 
   object$terms <- G$terms
@@ -1264,17 +1352,26 @@ bam.update <- function(b,data,chunk.size=10000) {
   rownames(X) <- NULL
   cnames <- names(b$coefficients)
 
-  ## now get the new data in model frame form...
+  AR.start <- NULL ## keep R checks happy
 
-  if ("(weights)"%in%names(b$model)) { 
+  ## now get the new data in model frame form...
+  getw <- "(weights)"%in%names(b$model)
+  getARs <- "(AR.start)"%in%names(b$model)
+  if (getw&&getARs) {
+    mf <- model.frame(gp$fake.formula,data,weights=weights,AR.start=AR.start,
+                      xlev=b$xlev,na.action=b$NA.action)
+    w <- mf[["(weights)"]]
+  } else if (getw) { 
     mf <- model.frame(gp$fake.formula,data,weights=weights,xlev=b$xlev,na.action=b$NA.action)
     w <- mf[["(weights)"]]
+  } else if (getARs) {
+    mf <- model.frame(gp$fake.formula,data,AR.start=AR.start,xlev=b$xlev,na.action=b$NA.action)
+    w <- rep(1,nrow(mf))
   } else {
     mf <- model.frame(gp$fake.formula,data,xlev=b$xlev,na.action=b$NA.action)
     w <- rep(1,nrow(mf))
   }
-
-
+  
   b$model <- rbind(b$model,mf) ## complete model frame --- old + new
 
   ## get response and offset...
@@ -1283,7 +1380,6 @@ bam.update <- function(b,data,chunk.size=10000) {
   if (is.null(off.col)) offset <- rep(0,nrow(mf)) else offset <-  mf[,off.col]
   y <-  mf[,attr(attr(b$model,"terms"),"response")] - offset
   
-
   ## update G
   b$G$y <- c(b$G$y,y)
   b$G$offset <- c(b$G$offset,offset)
@@ -1307,6 +1403,14 @@ bam.update <- function(b,data,chunk.size=10000) {
     row <- c(1,rep(1:m,rep(2,m))[-c(1,2*m)])
     weight <- c(1,rep(c(sd,ld),m-1))
     stop <- c(1,1:(m-1)*2+1)
+    if (!is.null(mf$"(AR.start)")) { ## need to correct the start of new AR sections...
+         ii <- which(mf$"(AR.start)"==TRUE)
+         if (length(ii)>0) {
+           if (ii[1]==1) ii <- ii[-1] ## first observation does not need any correction
+           weight[ii*2-2] <- 0 ## zero sub diagonal
+           weight[ii*2-1] <- 1 ## set leading diagonal to 1
+         }
+    }
    
     ## re-weight to independence....
     wX <- rwMatrix(stop,row,weight,wX)[-1,]
@@ -1328,7 +1432,8 @@ bam.update <- function(b,data,chunk.size=10000) {
     if (b$method=="GCV") scale <- -1 else scale = b$sig2
    
     fit <- magic(b$qrx$f,b$qrx$R,b$sp,b$G$S,b$G$off,L=b$G$L,lsp0=b$G$lsp0,rank=b$G$rank,
-               H=b$G$H,C=b$G$C,gamma=b$gamma,scale=scale,gcv=(scale<=0),
+               H=b$G$H,C= matrix(0,0,ncol(b$qrx$R)),##C=b$G$C,
+               gamma=b$gamma,scale=scale,gcv=(scale<=0),
                extra.rss=rss.extra,n.score=n)
  
     post <- magic.post.proc(b$qrx$R,fit,b$qrx$f*0+1) 
@@ -1364,7 +1469,8 @@ bam.update <- function(b,data,chunk.size=10000) {
      }
      
      if (b$AR1.rho!=0) { ## correct RE/ML score for AR1 transform
-       object$gcv.ubre <- object$gcv.ubre - (n-1)*log(ld)
+       df <- if (getARs) sum(b$model$"(AR.start)") else 1
+       object$gcv.ubre <- object$gcv.ubre - (n-df)*log(ld)
      }
 
      b$G$X <- b$qrx$R;b$G$dev.extra <- rss.extra
@@ -1385,7 +1491,8 @@ bam.update <- function(b,data,chunk.size=10000) {
     in.out <- list(sp=b$sp,scale=b$reml.scale)
     object <- gam(G=b$G,method=method,gamma=b$gamma,scale=scale,in.out=in.out) 
     if (b$AR1.rho!=0) { ## correct RE/ML score for AR1 transform
-       object$gcv.ubre <- object$gcv.ubre - (n-1)*log(ld)
+       df <- if (getARs) sum(b$model$"(AR.start)") else 1
+       object$gcv.ubre <- object$gcv.ubre - (n-df)*log(ld)
     }
     offset -> b$G$offset -> b$offset
     w -> b$G$w -> b$weights -> b$prior.weights; n -> b$G$n
