@@ -59,6 +59,53 @@ void read_mat(double *M,int *r,int*c,char *path) {
  fclose(mf);
 }
 
+void mgcv_tensor_mm(double *X,double *T,int *d,int *m,int *n) {
+/* Code for efficient production of row tensor product model matrices.
+   X contains rows of matrices to be producted. Contains m matrices,
+   d[i] is number of columns in ith matrix (which are stored in ascending 
+   order). Each column has n rows. T is the target matrix, with n rows
+   and \prod_i d[i] columns.   
+*/
+  int start, i,j,k, tp=1, xp=0,pd;
+  double *Xj,*Xj1,*Xk, *Tk,*p,*p1,*p2;
+  /*Rprintf("m = %d  n = %d  d = ",*m,*n);
+    for (i=0;i<*m;i++) Rprintf(" %d,",d[i]);*/
+  /* compute total columns in X, xp, and total columns in T, tp ... */
+  for (i=0;i<*m;i++) { xp += d[i];tp *= d[i];} 
+  Xk = X + (xp-d[*m-1]) * *n; /* start of last matrix in X */
+  Tk = T + (tp-d[*m-1]) * *n; /* start of last (filled) block in T */
+  /* initialize by putting final matrix in X into end block of T... */
+  p = Xk;p1 = Xk + *n * d[*m-1];p2 = Tk;
+  for (;p<p1;p++,p2++) *p2 = *p;
+  pd = d[*m-1]; /* cols in filled (end) block of T */
+  for (i = *m - 2;i>=0;i--) { /* work down through matrices stored in X */
+    Xk -= *n * d[i]; /* start of ith X matrix */
+    Xj = Xk;
+    start = tp - pd * d[i]; /* start column of target block in T */
+    p = T + start * *n; /* start location in T */
+    for (j=0;j<d[i];j++) { /* work through columns of ith X matrix */
+      p1 = Tk; /* start of T block to be multiplied by jth col of ith X matrix*/
+      Xj1 = Xj + *n; /* end of col j of ith X matrix */
+      /* following multiplies end T block by jth col of ith X matrix, storing 
+         result in appropriate T block... */
+      for (k=0;k<pd;k++) for (p2 = Xj;p2<Xj1;p2++,p1++,p++) *p = *p1 * *p2;
+      Xj += *n; /* start of col j+1 of ith X matrix */
+    }
+    pd *= d[i]; /* number of cols in filled in T block */
+    Tk = T + (tp - pd) * *n; /* start of filled in T block */
+  }  
+} /* mgcv_tensor_mm */
+
+void mgcv_tmm(SEXP x,SEXP t,SEXP D,SEXP M, SEXP N) {
+  /* wrapper for calling mgcv_tensor_mm using .Call */
+  double *X,*T;
+  int *d,*m,*n;
+  X = REAL(x);T=REAL(t);
+  d = INTEGER(D);
+  m = INTEGER(M);
+  n = INTEGER(N);
+  mgcv_tensor_mm(X,T,d,m,n);
+}
 
 void mgcv_mmult0(double *A,double *B,double *C,int *bt,int *ct,int *r,int *c,int *n)
 /* This code doesn't rely on the BLAS...
@@ -171,6 +218,8 @@ void mgcv_mmult(double *A,double *B,double *C,int *bt,int *ct,int *r,int *c,int
 		B, &lda,C, &ldb,&beta, A, &ldc);
 } /* end mgcv_mmult */
 
+
+
 SEXP mgcv_pmmult2(SEXP b, SEXP c,SEXP bt,SEXP ct, SEXP nthreads) {
 /* parallel matrix multiplication using .Call interface */
   double *A,*B,*C;
